<?php

namespace Drupal\voiceflow_index\Form;

use Drupal\Core\Database\Database;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Settings form for the Voiceflow Index.
 */
class VoiceflowIndexSettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new VoiceflowIndexSettingsForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['voiceflow_index.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'voiceflow_index_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('voiceflow_index.settings');

    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    $options = [];
    foreach ($content_types as $type) {
      $options[$type->id()] = $type->label();
    }

    $form['content_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Select content types to index to Voiceflow'),
      '#options' => $options,
      '#default_value' => $config->get('content_types') ?? [],
      '#description' => $this->t('Disabling a previously enabled content type, will be reset the "Voiceflow Index" values stored in database of nodes.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable('voiceflow_index.settings');

    // Get which content types were previously configured.
    $config_content_types = $config->get('content_types') ?? [];
    // Update the configuration with the selected types.
    $selected_types = array_filter($form_state->getValue('content_types')) ?? [];
    $config->set('content_types', $selected_types)->save();

    // Add the field "voiceflow_index" to the new selected content types.
    foreach ($selected_types as $bundle) {
      if (!in_array($bundle, $config_content_types)) {
        FieldConfig::create([
          'field_storage' => FieldStorageConfig::loadByName('node', 'voiceflow_index'),
          'bundle' => $bundle,
          'label' => $this->t('Index to Voiceflow'),
          'translatable' => FALSE,
          'default_value' => [TRUE],
        ])->save();
      }
    }

    foreach (array_diff($config_content_types, $selected_types) as $bundle) {
      // Remove the field "voiceflow_index" to the unselected content types.
      if ($field = FieldConfig::loadByName('node', $bundle, 'voiceflow_index')) {
        $field->delete();
      }
      // Remove "voiceflow_index" storage in unselected content types.
      $connection = Database::getConnection();
      $connection->delete('node__voiceflow_index')
        ->condition('bundle', $bundle)
        ->execute();
    }

    parent::submitForm($form, $form_state);
  }

}
